unit uMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls, DB, ADODB, Grids, DBGrids{, ADODB_TLB};

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    btnLogueo: TButton;
    btnDeslogueo: TButton;
    edtLlave: TEdit;
    edtEmpresa: TEdit;
    edtUsuario: TEdit;
    edtPassword: TEdit;
    btnCrearStr: TButton;
    btnCargarXML: TButton;
    btnIngresarPedidos: TButton;
    btnGrabarXML: TButton;
    PageControl1: TPageControl;
    tabConfig: TTabSheet;
    tabEncab: TTabSheet;
    tabRenglones: TTabSheet;
    tabResult: TTabSheet;
    ADODSEncab: TADODataSet;
    ADODSRenglon: TADODataSet;
    ADODSResult: TADODataSet;
    dsEncab: TDataSource;
    dsRenglon: TDataSource;
    dsResult: TDataSource;
    DBGrEncab: TDBGrid;
    DBGrRenglon: TDBGrid;
    DBGrResult: TDBGrid;
    ckbLPrecios: TCheckBox;
    ckbTalonario: TCheckBox;
    ckbClieInhab: TCheckBox;
    ckbCredito: TCheckBox;
    btnSalir: TButton;
    procedure FormCreate(Sender: TObject);
    procedure btnLogueoClick(Sender: TObject);
    procedure btnDeslogueoClick(Sender: TObject);
    procedure btnCrearStrClick(Sender: TObject);
    procedure btnIngresarPedidosClick(Sender: TObject);
    procedure btnCargarXMLClick(Sender: TObject);
    procedure btnGrabarXMLClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnSalirClick(Sender: TObject);
  private
    fPedidos : OleVariant;
    rsEncabezado : _Recordset;
    rsRenglones  : _Recordset;
    rsResultado  : _Recordset;
    DirectorioDatos : string;
    function GetPedidos: OleVariant;
    function CrearPedidos: OleVariant;
    function LoguarmeATango: Boolean;
    function ExistenArchivosXML() : Boolean;
    procedure DesloguearmeDeTango;
    procedure CrearEstructurasPedidos;
    procedure IngresarPedidos;
    procedure HabilitarControles(Estado: boolean);
    procedure CopiarDatosRecordSet(rsOrigen, rsDestino : _RecordSet);
    procedure GrabarDatosXML(rs: _RecordSet; Archivo : string);
  public
    property Pedidos : OleVariant read GetPedidos write fPedidos;
  end;

var
  Form1: TForm1;

implementation

uses ComObj, ADOInt;

{$R *.dfm}
const
  XMLEnc    = 'PedEnc.xml';
  XMLRen    = 'PedRen.xml';
  XMLResult = 'PedResult.xml';
  XMLStrEnc = 'PedEnc_Str.xml';
  XMLStrRen = 'PedRen_Str.xml';

procedure TForm1.CrearEstructurasPedidos;
var
  R : integer;
begin
  rsEncabezado := nil;
  rsRenglones  := nil;
  R := Pedidos.Str_Pedido(rsEncabezado, rsRenglones);
  if R <> 0 then
    ShowMessage('Error al crear las estructuras.')
  else begin
    GrabarDatosXML(rsEncabezado, IncludeTrailingPathDelimiter(DirectorioDatos) + XMLStrEnc);
    GrabarDatosXML(rsRenglones,  IncludeTrailingPathDelimiter(DirectorioDatos) + XMLStrRen);
  end;
end;

function TForm1.CrearPedidos : OleVariant;
begin
  result := Unassigned;
  try
    result := CreateComObject(ProgIdToClassId('Com_Gv.Pedidos')) as IDispatch;
  except
    on E: Exception do
      ShowMessage('No pudo crear el objeto pedidos, verificar registracin ('+E.Message+').');
  end;
end;

procedure TForm1.DesloguearmeDeTango;
begin
  if Pedidos.TgLog.wbIsLogged then
    Pedidos.TgLog.Logoff;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  fPedidos := Unassigned;
  rsEncabezado := nil;
  rsRenglones  := nil;
  rsResultado  := nil;
  DirectorioDatos := ExtractFilePath(Application.ExeName);
  HabilitarControles(false);
  PageControl1.ActivePageIndex := 0;
end;

function TForm1.GetPedidos: OleVariant;
begin
  if VarIsEmpty(fPedidos) then
    fPedidos := CrearPedidos;
  Result := fPedidos;
end;

procedure TForm1.HabilitarControles(Estado: boolean);
begin
  btnCrearStr.Enabled        := Estado;
  btnCargarXML.Enabled       := Estado;
  btnGrabarXML.Enabled       := Estado;
  btnIngresarPedidos.Enabled := Estado;
  PageControl1.Enabled       := Estado;
end;

procedure TForm1.IngresarPedidos;
var
  R : integer;
begin
  if assigned(rsEncabezado) and assigned(rsRenglones) and
     (rsEncabezado.RecordCount > 0) and (rsRenglones.RecordCount > 0) then begin
    try
      R := Pedidos.Add_Pedidos(rsEncabezado, rsRenglones, rsResultado, ckbCredito.Checked,
           ckbClieInhab.Checked, ckbTalonario.Checked, ckbLPrecios.Checked, true);
      if R <> 0 then
        ShowMessage('Error al ingresar pedidos.')
      else
        ADODSResult.Recordset := rsResultado;
    except
      on E : Exception do
        ShowMessage('Fallo el ingreso de pedidos ('+E.Message+').');
    end;
  end
  else
    ShowMessage('No hay pedidos cargados.');
end;

function TForm1.LoguarmeATango : boolean;
begin
  if Pedidos.TgLog.wbIsLogged then
    DesloguearmeDeTango;
  Pedidos.TgLog.wsNroLlave      := edtLlave.Text;
  Pedidos.TgLog.wsNombreEmpresa := edtEmpresa.Text;
  Pedidos.TgLog.wsNombreUsuario := edtUsuario.Text;
  Pedidos.TgLog.Set_wsPassword(edtPassword.Text);
  Pedidos.TgLog.wsTitleApp      := 'Demo XTango Pedidos';
  Pedidos.TgLog.inHandleApp     := Application.Handle;
  if not Pedidos.TgLog.Login('', '') then begin
    ShowMessage('No se pudo loguear al sistema.');
    result := false;
  end
  else begin
    edtLlave.Text    := Pedidos.TgLog.wsNroLlave;
    edtEmpresa.Text  := Pedidos.TgLog.wsNombreEmpresa;
    edtUsuario.Text  := Pedidos.TgLog.wsNombreUsuario;
    result := true;
  end;
end;

procedure TForm1.btnLogueoClick(Sender: TObject);
begin
  if LoguarmeATango then begin
    refresh;
    btnCrearStr.Click;
    refresh;
    HabilitarControles(true);
  end
  else
    HabilitarControles(false);
end;

procedure TForm1.btnDeslogueoClick(Sender: TObject);
begin
  DesloguearmeDeTango;
  HabilitarControles(false);
end;

procedure TForm1.btnCrearStrClick(Sender: TObject);
begin
  ADODSEncab.DisableControls;
  ADODSRenglon.DisableControls;

  CrearEstructurasPedidos;

  ADODSEncab.Recordset   := rsEncabezado;
  ADODSRenglon.Recordset := rsRenglones;
  ADODSResult.Recordset  := nil;
  ADODSEncab.EnableControls;
  ADODSRenglon.EnableControls;
end;

procedure TForm1.btnIngresarPedidosClick(Sender: TObject);
begin
  IngresarPedidos;
  PageControl1.ActivePageIndex := 3;
end;

procedure TForm1.CopiarDatosRecordSet(rsOrigen, rsDestino: _RecordSet);
var
  i : integer;
  N : string;
begin
  if assigned(rsDestino) and (rsDestino.RecordCount > 0) then begin
    rsDestino.MoveFirst;
    while not rsDestino.EOF do begin
      rsDestino.Delete(adAffectCurrent);
      rsDestino.MoveNext;
    end;
  end;
  if assigned(rsDestino) and (rsOrigen.RecordCount > 0) then begin
    rsOrigen.MoveFirst;
    while not rsOrigen.EOF do begin
      rsDestino.AddNew(EmptyParam, EmptyParam);
      for i := 0 to rsDestino.Fields.Count -1 do begin
        N := rsDestino.Fields.Item[i].Name;
        rsDestino.Fields.Item[N].Value := rsOrigen.Fields.Item[N].Value;
      end;
      rsDestino.Update(EmptyParam, EmptyParam);
      rsOrigen.MoveNext;
    end;
    rsDestino.MoveFirst;
  end;
end;


function TForm1.ExistenArchivosXML() : Boolean;
begin
  if not FileExists(IncludeTrailingPathDelimiter(DirectorioDatos) + XMLEnc) or
     not FileExists(IncludeTrailingPathDelimiter(DirectorioDatos) + XMLRen)  or
     not FileExists(IncludeTrailingPathDelimiter(DirectorioDatos) + XMLResult) then
    Result := False
  else
    Result := True;
end;

procedure TForm1.btnCargarXMLClick(Sender: TObject);
begin

  if Not ExistenArchivosXML then begin
    ShowMessage('No se puede Cargar - Archivos XML no encontrados');
    exit;
  end;

  try
      // trae los datos del XML y los trae al global

      rsEncabezado := CoRecordset.Create;
      rsEncabezado.Open(IncludeTrailingPathDelimiter(DirectorioDatos) + XMLEnc, EmptyParam, adOpenDynamic, adOpenDynamic, 0);
      ADODSEncab.Recordset := rsEncabezado;


      rsRenglones := CoRecordset.Create;
      rsRenglones.Open(IncludeTrailingPathDelimiter(DirectorioDatos) + XMLRen, EmptyParam, adOpenDynamic, adOpenDynamic, 0);
      ADODSRenglon.Recordset := rsRenglones;


      rsResultado := CoRecordset.Create;
      rsResultado.Open(IncludeTrailingPathDelimiter(DirectorioDatos) + XMLResult, EmptyParam, adOpenDynamic, adOpenDynamic, 0);
      ADODSResult.Recordset := rsResultado;

      // Libera estructuras

  except
   on E : Exception do
     MessageDlg('Error al cargar los datos (' + E.Message + ')', mtWarning, [mbOK], 0);
  end;
end;

procedure TForm1.GrabarDatosXML(rs: _RecordSet; Archivo : string);
begin
  if assigned(rs) then begin
    if FileExists(Archivo) then
      DeleteFile(Archivo);
    rs.Save(Archivo, adPersistXML);  
  end;
end;

procedure TForm1.btnGrabarXMLClick(Sender: TObject);
begin
  ADODSEncab.SaveToFile   (IncludeTrailingPathDelimiter(DirectorioDatos) + XMLEnc, pfXML);
  ADODSRenglon.SaveToFile (IncludeTrailingPathDelimiter(DirectorioDatos) + XMLRen , pfXML);
  ADODSResult.SaveToFile  (IncludeTrailingPathDelimiter(DirectorioDatos) + XMLResult, pfXML);
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if not VarIsEmpty(Pedidos) then begin
    DesloguearmeDeTango;
    Pedidos := Unassigned;
  end;
  rsEncabezado := nil;
  rsRenglones  := nil;
  rsResultado  := nil;
end;

procedure TForm1.btnSalirClick(Sender: TObject);
begin
  Close;
end;

end.
